
--
-- jmd / issue 4655 add saved_search_tbl plus views to the database
--

-- DROP TABLE saved_seatch_tbl CASCADE;

CREATE TABLE saved_search_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    bearbeiter_id INTEGER NOT NULL,
    name VARCHAR(128) NOT NULL,
    formed_version TEXT NOT NULL,
    search TEXT NOT NULL,

    -- Use ON DELETE CASCADE for the foreign key constraint on
    -- bearbeiter_id so that we do not have to explicitly delete a
    -- user's searches when the user account is deleted. The searches
    -- are not important enough to warrant explicit confirmation from
    -- the admin before they're deleted when the user account is
    -- deleted.
    FOREIGN KEY (bearbeiter_id) REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE
);

CREATE OR REPLACE VIEW saved_search_tbl_view AS
SELECT saved_search_tbl.id,
       saved_search_tbl.bearbeiter_id,
       saved_search_tbl.name,
       saved_search_tbl.formed_version,
       saved_search_tbl.search
FROM saved_search_tbl
WHERE saved_search_tbl.bearbeiter_id = getuserid();

GRANT SELECT ON saved_search_tbl_view TO GROUP :cm_group;
GRANT SELECT ON saved_search_tbl_view TO GROUP :admin_group;



-- Includes the stored procedure to delete a saved search
CREATE OR REPLACE FUNCTION delete_savedsearch_ds(tid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
BEGIN
    DELETE FROM saved_search_tbl
    WHERE id = tid AND getuserid() = bearbeiter_id;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_savedsearch_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_savedsearch_ds(integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION delete_savedsearch_ds(integer) TO GROUP :admin_group;

-- Include the stored rename procedure
CREATE OR REPLACE FUNCTION rename_savedsearch_ds(tid integer, name_new varchar(128))
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
BEGIN
    UPDATE saved_search_tbl SET name = name_new
    WHERE id = tid AND getuserid() = bearbeiter_id;
END;
$$ ;

REVOKE ALL    ON FUNCTION rename_savedsearch_ds(integer, varchar(128)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION rename_savedsearch_ds(integer, varchar(128)) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION rename_savedsearch_ds(integer, varchar(128)) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION create_saved_search_ds(new_name VARCHAR(128), new_formed_version TEXT, new_search TEXT) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
DECLARE
     bid integer;
BEGIN
     bid = getuserid();
     INSERT INTO saved_search_tbl(bearbeiter_id, name, formed_version, search)
     VALUES (bid, new_name, new_formed_version, new_search);
END;
$$;

-- Include the stored create procedure
REVOKE ALL    ON FUNCTION create_saved_search_ds(VARCHAR(128), TEXT, TEXT) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_saved_search_ds(VARCHAR(128), TEXT, TEXT) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_saved_search_ds(VARCHAR(128), TEXT, TEXT) TO GROUP :admin_group;
