BEGIN;

-- 1.0.2 -> 1.0.3
-- issue1801: Fügt neue Felder in die Fallakte ein

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('1.0.3/02-gruppenagebote-alter.sql', current_timestamp);


ALTER TABLE rg_kjp_gruppenangebote_tbl
  ADD COLUMN kjp_grp_alter2 INTEGER DEFAULT -1,
  ADD COLUMN kjp_grp_alter3 INTEGER DEFAULT -1,
  ADD FOREIGN KEY (kjp_grp_alter2) REFERENCES kjp_grp_alter_tbl (id),
  ADD FOREIGN KEY (kjp_grp_alter3) REFERENCES kjp_grp_alter_tbl (id);

DROP VIEW rg_kjp_gruppenangebote_tbl_view CASCADE;

CREATE OR REPLACE VIEW rg_kjp_gruppenangebote_tbl_view
AS SELECT 
     rg_kjp_gruppenangebote_tbl.id AS id,
     rg_kjp_gruppenangebote_tbl.master_id AS master_id,
     rg_kjp_gruppenangebote_tbl.uuid_id AS uuid_id,
     rg_kjp_gruppenangebote_tbl.kjp_grp_art_angebot AS kjp_grp_art_angebot,
     rg_kjp_gruppenangebote_tbl.kjp_grp_anzahl_teilnehmer_maennlich AS kjp_grp_anzahl_teilnehmer_maennlich,
     rg_kjp_gruppenangebote_tbl.kjp_grp_anzahl_teilnehmer_weiblich AS kjp_grp_anzahl_teilnehmer_weiblich,
     rg_kjp_gruppenangebote_tbl.kjp_grp_alter AS kjp_grp_alter,
     rg_kjp_gruppenangebote_tbl.kjp_grp_alter2 AS kjp_grp_alter2,
     rg_kjp_gruppenangebote_tbl.kjp_grp_alter3 AS kjp_grp_alter3,
     rg_kjp_gruppenangebote_tbl.kjp_grp_weitere_angaben AS kjp_grp_weitere_angaben
FROM rg_kjp_gruppenangebote_tbl
    INNER JOIN master_tbl
    ON         rg_kjp_gruppenangebote_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');



CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_update_rule AS ON UPDATE TO rg_kjp_gruppenangebote_tbl_view
DO INSTEAD UPDATE rg_kjp_gruppenangebote_tbl SET
    kjp_grp_art_angebot = NEW.kjp_grp_art_angebot,
    kjp_grp_anzahl_teilnehmer_maennlich = NEW.kjp_grp_anzahl_teilnehmer_maennlich,
    kjp_grp_anzahl_teilnehmer_weiblich = NEW.kjp_grp_anzahl_teilnehmer_weiblich,
    kjp_grp_alter = NEW.kjp_grp_alter,
    kjp_grp_alter2 = NEW.kjp_grp_alter2,
    kjp_grp_alter3 = NEW.kjp_grp_alter3,
    kjp_grp_weitere_angaben = NEW.kjp_grp_weitere_angaben
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_delete_rule AS ON DELETE TO rg_kjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_insert_rule AS ON INSERT TO rg_kjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_kjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_kjp_gruppenangebote_tbl_view TO GROUP :admin_group;


CREATE OR REPLACE VIEW rg_kjp_gruppenangebote_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    kjp_grp_art_angebot,
    kjp_grp_anzahl_teilnehmer_maennlich,
    kjp_grp_anzahl_teilnehmer_weiblich,
    kjp_grp_alter,
    kjp_grp_alter2,
    kjp_grp_alter3,
    kjp_grp_weitere_angaben FROM rg_kjp_gruppenangebote_tbl_view;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_anonym_delete_rule AS ON DELETE TO rg_kjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_anonym_insert_rule AS ON INSERT TO rg_kjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kjp_gruppenangebote_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_kjp_gruppenangebote_tbl_anonym_view TO GROUP :admin_group;




-- Finish
SET ROLE postgres;

COMMIT;
