BEGIN;

--
-- Change choice fields kjg_grp_alter and nichtkjg_grp_alter into checkbox
-- fields. (issue 3281)
--

-- Disable trigger so the last accessed time is not changed
ALTER TABLE rg_kjp_gruppenangebote_tbl DISABLE TRIGGER USER;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

-- History
INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('1.3.1/02_alter.sql', current_timestamp);

-- Add new columns for options
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD COLUMN kjp_alter1 INTEGER DEFAULT -1;
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD COLUMN kjp_alter2 INTEGER DEFAULT -1;
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD COLUMN kjp_alter3 INTEGER DEFAULT -1;

ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD COLUMN nichtkjp_alter1 INTEGER DEFAULT -1;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD COLUMN nichtkjp_alter2 INTEGER DEFAULT -1;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD COLUMN nichtkjp_alter3 INTEGER DEFAULT -1;

-- Add constraints to new options
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter1) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter2) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter3) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter1) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter2) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter3) REFERENCES ha_sprachkenntnis_arabisch_tbl (id);


-- Update the new values from the old options
UPDATE rg_kjp_gruppenangebote_tbl SET kjp_alter1 = 1 
WHERE kjp_grp_alter = 0 OR  kjp_grp_alter2 = 0 OR kjp_grp_alter3 = 0;
UPDATE rg_kjp_gruppenangebote_tbl SET kjp_alter2 = 1 
WHERE kjp_grp_alter = 1 OR  kjp_grp_alter2 = 1 OR kjp_grp_alter3 = 1;
UPDATE rg_kjp_gruppenangebote_tbl SET kjp_alter3 = 1 
WHERE kjp_grp_alter = 2 OR  kjp_grp_alter2 = 2 OR kjp_grp_alter3 = 2;

UPDATE rg_nichtkjp_gruppenangebote_tbl SET nichtkjp_alter1 = 1 
WHERE nichtkjp_grp_alter = 0 OR  nichtkjp_grp_alter2 = 0 OR nichtkjp_grp_alter3 = 0;
UPDATE rg_nichtkjp_gruppenangebote_tbl SET nichtkjp_alter2 = 1 
WHERE nichtkjp_grp_alter = 1 OR  nichtkjp_grp_alter2 = 1 OR nichtkjp_grp_alter3 = 1;
UPDATE rg_nichtkjp_gruppenangebote_tbl SET nichtkjp_alter3 = 1 
WHERE nichtkjp_grp_alter = 2 OR  nichtkjp_grp_alter2 = 2 OR nichtkjp_grp_alter3 = 2;

-- Delete the views.
DROP VIEW rg_kjp_gruppenangebote_tbl_view CASCADE;
DROP VIEW rg_nichtkjp_gruppenangebote_tbl_view CASCADE;

-- Clean up old columns
ALTER TABLE rg_kjp_gruppenangebote_tbl DROP COLUMN kjp_grp_alter;
ALTER TABLE rg_kjp_gruppenangebote_tbl DROP COLUMN kjp_grp_alter2;
ALTER TABLE rg_kjp_gruppenangebote_tbl DROP COLUMN kjp_grp_alter3;

ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP COLUMN nichtkjp_grp_alter;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP COLUMN nichtkjp_grp_alter2;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP COLUMN nichtkjp_grp_alter3;

-- Remove old option table
DROP TABLE kjp_grp_alter_tbl CASCADE;

-- Recreate the views
CREATE OR REPLACE VIEW rg_kjp_gruppenangebote_tbl_view
AS SELECT 
     rg_kjp_gruppenangebote_tbl.id AS id,
     rg_kjp_gruppenangebote_tbl.master_id AS master_id,
     rg_kjp_gruppenangebote_tbl.uuid_id AS uuid_id,
     rg_kjp_gruppenangebote_tbl.kjp_grp_art_angebot AS kjp_grp_art_angebot,
     rg_kjp_gruppenangebote_tbl.kjp_grp_dauer_stunden AS kjp_grp_dauer_stunden,
     rg_kjp_gruppenangebote_tbl.kjp_grp_anzahl_teilnehmer_maennlich AS kjp_grp_anzahl_teilnehmer_maennlich,
     rg_kjp_gruppenangebote_tbl.kjp_grp_anzahl_teilnehmer_weiblich AS kjp_grp_anzahl_teilnehmer_weiblich,
     rg_kjp_gruppenangebote_tbl.kjp_alter1 AS kjp_alter1,
     rg_kjp_gruppenangebote_tbl.kjp_alter2 AS kjp_alter2,
     rg_kjp_gruppenangebote_tbl.kjp_alter3 AS kjp_alter3,
     rg_kjp_gruppenangebote_tbl.kjp_grp_weitere_angaben AS kjp_grp_weitere_angaben
FROM rg_kjp_gruppenangebote_tbl
    INNER JOIN master_tbl
    ON         rg_kjp_gruppenangebote_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_update_rule AS ON UPDATE TO rg_kjp_gruppenangebote_tbl_view
DO INSTEAD UPDATE rg_kjp_gruppenangebote_tbl SET
    kjp_grp_art_angebot = NEW.kjp_grp_art_angebot,
    kjp_grp_dauer_stunden = NEW.kjp_grp_dauer_stunden,
    kjp_grp_anzahl_teilnehmer_maennlich = NEW.kjp_grp_anzahl_teilnehmer_maennlich,
    kjp_grp_anzahl_teilnehmer_weiblich = NEW.kjp_grp_anzahl_teilnehmer_weiblich,
    kjp_alter1 = NEW.kjp_alter1,
    kjp_alter2 = NEW.kjp_alter2,
    kjp_alter3 = NEW.kjp_alter3,
    kjp_grp_weitere_angaben = NEW.kjp_grp_weitere_angaben
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_delete_rule AS ON DELETE TO rg_kjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_insert_rule AS ON INSERT TO rg_kjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_kjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_kjp_gruppenangebote_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_kjp_gruppenangebote_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    kjp_grp_art_angebot,
    kjp_grp_dauer_stunden,
    kjp_grp_anzahl_teilnehmer_maennlich,
    kjp_grp_anzahl_teilnehmer_weiblich,
    kjp_alter1,
    kjp_alter2,
    kjp_alter3,
    kjp_grp_weitere_angaben FROM rg_kjp_gruppenangebote_tbl_view;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_anonym_delete_rule AS ON DELETE TO rg_kjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kjp_gruppenangebote_tbl_anonym_insert_rule AS ON INSERT TO rg_kjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kjp_gruppenangebote_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_kjp_gruppenangebote_tbl_anonym_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_nichtkjp_gruppenangebote_tbl_view
AS SELECT 
     rg_nichtkjp_gruppenangebote_tbl.id AS id,
     rg_nichtkjp_gruppenangebote_tbl.master_id AS master_id,
     rg_nichtkjp_gruppenangebote_tbl.uuid_id AS uuid_id,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_grp_art_angebot AS nichtkjp_grp_art_angebot,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_grp_dauer_stunden AS nichtkjp_grp_dauer_stunden,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_grp_anzahl_teilnehmer_maennlich AS nichtkjp_grp_anzahl_teilnehmer_maennlich,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_grp_anzahl_teilnehmer_weiblich AS nichtkjp_grp_anzahl_teilnehmer_weiblich,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_alter1 AS nichtkjp_alter1,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_alter2 AS nichtkjp_alter2,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_alter3 AS nichtkjp_alter3,
     rg_nichtkjp_gruppenangebote_tbl.nichtkjp_grp_weitere_angaben AS nichtkjp_grp_weitere_angaben
FROM rg_nichtkjp_gruppenangebote_tbl
    INNER JOIN master_tbl
    ON         rg_nichtkjp_gruppenangebote_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE rg_nichtkjp_gruppenangebote_tbl_update_rule AS ON UPDATE TO rg_nichtkjp_gruppenangebote_tbl_view
DO INSTEAD UPDATE rg_nichtkjp_gruppenangebote_tbl SET
    nichtkjp_grp_art_angebot = NEW.nichtkjp_grp_art_angebot,
    nichtkjp_grp_dauer_stunden = NEW.nichtkjp_grp_dauer_stunden,
    nichtkjp_grp_anzahl_teilnehmer_maennlich = NEW.nichtkjp_grp_anzahl_teilnehmer_maennlich,
    nichtkjp_grp_anzahl_teilnehmer_weiblich = NEW.nichtkjp_grp_anzahl_teilnehmer_weiblich,
    nichtkjp_alter1 = NEW.nichtkjp_alter1,
    nichtkjp_alter2 = NEW.nichtkjp_alter2,
    nichtkjp_alter3 = NEW.nichtkjp_alter3,
    nichtkjp_grp_weitere_angaben = NEW.nichtkjp_grp_weitere_angaben
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_nichtkjp_gruppenangebote_tbl_delete_rule AS ON DELETE TO rg_nichtkjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_nichtkjp_gruppenangebote_tbl_insert_rule AS ON INSERT TO rg_nichtkjp_gruppenangebote_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_nichtkjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_nichtkjp_gruppenangebote_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_nichtkjp_gruppenangebote_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_nichtkjp_gruppenangebote_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    nichtkjp_grp_art_angebot,
    nichtkjp_grp_dauer_stunden,
    nichtkjp_grp_anzahl_teilnehmer_maennlich,
    nichtkjp_grp_anzahl_teilnehmer_weiblich,
    nichtkjp_alter1,
    nichtkjp_alter2,
    nichtkjp_alter3,
    nichtkjp_grp_weitere_angaben FROM rg_nichtkjp_gruppenangebote_tbl_view;

CREATE OR REPLACE RULE rg_nichtkjp_gruppenangebote_tbl_anonym_delete_rule AS ON DELETE TO rg_nichtkjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_nichtkjp_gruppenangebote_tbl_anonym_insert_rule AS ON INSERT TO rg_nichtkjp_gruppenangebote_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_nichtkjp_gruppenangebote_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_nichtkjp_gruppenangebote_tbl_anonym_view TO GROUP :admin_group;

-- Reactivate the role
SET ROLE postgres;
-- Reactivate the triggers
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ENABLE TRIGGER USER;
ALTER TABLE rg_kjp_gruppenangebote_tbl ENABLE TRIGGER USER;

COMMIT;

