# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2014 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD.
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

import formencode
from formencode.validators import String

from pylons import url
from pylons.controllers.util import redirect

from mpulsweb.lib.validators import BaseFormValidator
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.base import session

from mpulsweb.controllers.case import CaseController as MpulsCaseController


log = logging.getLogger(__name__)


class CaseController(MpulsCaseController):

    def _get_validator(self):
        '''Set specific validator'''
        return BaseFormValidator(datum_begin_kooperation=String(not_empty=True),
                                 datum_ende_kooperation=String())

    @checkRole(('admin', 'cm', 'pt_dlr'))
    def select(self, id, confirmed=0):
        """Overwrite base method to redirect to first form page."""
        self._loadCase(id)
        navigation = session['navigation.tree']
        redirect(url(controller="navigation", action="select_branch",
                     id=navigation.first_page.key))

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
