# -*- coding: utf-8 -*-
#
# Copyright 2018 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Ludwig Reiter <ludwig.reiter@intevation.de>


from mpulsweb.controllers.appointment import (
    AppointmentController as MpulsAppointmentController
)
from mpulsweb.lib.base import render, c
from mpulsweb.lib.helpers import url
from mpulsweb.lib.security import checkRole, hasRole
from mpulsweb.lib.translation import _
from mpulsweb.model.appointment import (
    GlobalAppointmentOverview,
    CaseAppointmentOverview
)

from jmdweb.controllers.mpuls import get_jmd_reminders


class AppointmentController(MpulsAppointmentController):

    @checkRole(('admin', 'cm', 'pt_dlr'))
    def overview(self):
        c.globalappointmentlist = GlobalAppointmentOverview(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        c.appointmentlist = CaseAppointmentOverview(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        return render('/appointments/appointment_overview.mako')

    @checkRole(('admin', 'cm', 'pt_dlr'))
    def reminder_overview(self):
        c.remindlist = get_jmd_reminders(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        c.show_set_done = True
        c.set_done_backlink = "reminder_overview"
        return render('/appointments/reminder_overview.mako')

    def _get_use_trigger_and_done(self):
        use_trigger_and_done = True
        if hasRole('admin'):
            use_trigger_and_done = False
            c.show_set_done_title = True
        return use_trigger_and_done

    def _handle_backlink(self, backlink):
        if backlink != "reminder_overview":
            return super(AppointmentController, self)._handle_backlink(backlink)

        return (_(u"The appointment was successfully set done.")
                + " "
                + _("Please click OK to continue to the reminder overview."),
                url(controller='appointment', action="reminder_overview"))
