# -*- coding: utf-8 -*-
#
# Copyright 2015 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

import mpulsweb.lib.sqlexpr as E

from mpulsweb.lib.searchexpr import Expr, qualified_name

from pylons import app_globals as g

class BoolGroup(Expr):
    """ Special expression to search for cases where at least one
        Value in a group is set.

        This class could be pretty and general.
        Damn you bezugsperson_q !
        """

    def __init__(self, groupname, value):
        self.groupname = groupname
        self.value = value
        formed_group = g.formedTree.findByName(groupname)
        self.child_fields = [child.getName()
                             for child in formed_group.children]

    def __repr__(self):
        return "BoolGroup(%r, %r)" % (self.groupname, self.value)

    def to_simple_format(self):
        return {
            'cls': self.__class__.__name__,
            'attrs': {"groupname": self.groupname,
                      "value": self.value
                      },
        }

    def sql_expr(self):
        if self.value is None:
            return E.TRUE

        if self.value == 1:
            return E.NAryOr([E.EQ(E.Field(qualified_name(child_field)),
                                          E.Const(self.value))
                             for child_field in self.child_fields
                             if child_field != "bezugsperson_q"])

        return E.NAryOr([E.NAryAnd(
                         [E.NAryOr([E.EQ(E.Field(qualified_name(child_field)),
                                      E.Const(self.value)),
                                    E.IS_NULL(
                                      E.Field(qualified_name(child_field)))])
                          for child_field in self.child_fields
                          if child_field != "bezugsperson_q"]),
                          E.EQ(E.Field("bezugsperson_q"), E.Const(1))])

    def retrieve_fields(self):
        if self.value:
            for child_name in self.child_fields:
                yield qualified_name(child_name)

    def form_key(self):
        return self.groupname

