# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Roland Geider <roland.geider@intevation.de>
# Ludwig Reiter <ludwig.reiter@intevation.de>


import logging

from pylons.controllers.util import redirect

from mpulsweb.lib.security import checkRole
from mpulsweb.lib.base import BaseController, request, c, render, \
    render_htmlfill, restrict_to_POST, session
from mpulsweb.lib.translation import _
from mpulsweb.lib.helpers import url
from mpulsweb.lib.validators import RenameSavedSearchForm
from mpulsweb.model.savedsearch import SavedSearch

from mpulsweb.controllers.case_overview import put_session_search


log = logging.getLogger(__name__)


class SavedsearchController(BaseController):

    @checkRole(('cm', 'admin'))
    def overview(self):
        # Return a list of saved searches with actions
        c.saved_searches = SavedSearch.list_saved_searches()
        overview = render('/savedsearch/overview.mako')
        return overview

    def delete(self, id, confirmed=False):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)

        if confirmed == 1:
            entry = SavedSearch.get_saved_search_by_id(int(id))
            if entry:
                entry.delete()
            return redirect(url(controller='savedsearch', action='overview'))
        else:
            c.confirm_for = _(u"Delete saved search?")
            c.question = _(u"Do you want to delete this saved search?\n"
                           u"It will be irrevocably lost.")
            c.url_yes = url(controller='savedsearch',
                            action='delete',
                            id=id,
                            confirmed=1)
            c.url_no = url(controller='savedsearch', action='overview')
            return render('/savedsearch/dialogs/confirm_delete_savedsearch.mako')

    def load(self, id):
        id = self._checkInt(id)
        entry = SavedSearch.get_saved_search_by_id(id)
        if entry is None:
            redirect(url(controller='savedsearch', action='overview'))
        else:
            put_session_search(entry.get_searchexpr())
            return redirect(url(controller='case_overview',
                                action='overview'))

    def rename(self, id):
        id = self._checkInt(id)
        entry = SavedSearch.get_saved_search_by_id(id)
        if entry is None:
            redirect(url(controller='savedsearch', action='overview'))
        else:
            c.entry = entry
            entry_data = {}
            entry_data['id'] = entry.id
            entry_data['name'] = entry.name
            form = render('/savedsearch/rename.mako')
            return render_htmlfill(form, defaults=entry_data,
                                   auto_insert_errors=False, errors={})

    @restrict_to_POST
    def renameAction(self):
        validator = RenameSavedSearchForm()
        id = self._checkInt(request.params.getone('id'))
        try:
            form_result = validator.to_python(request.params)
            entry = SavedSearch.get_saved_search_by_id(id)
            if entry:
                entry.rename(form_result['name'])
        except:
            log.exception("cannot rename saved search")
        redirect(url(controller='savedsearch', action='overview'))
