# -*- coding: utf-8 -*-
import logging
import formencode

from pylons import request, tmpl_context as c, app_globals as g

from mpulsweb.lib.base import BaseController, render, render_htmlfill, \
    restrict_to_POST
from mpulsweb.lib.helpers import url
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.translation import _
from mpulsweb.lib.validators import EditSettingsForm, MaxMinInt
from mpulsweb.model.agencysettings import Agency


log = logging.getLogger(__name__)


class SettingsController(BaseController):

    @checkRole('admin')
    def index(self):
        return self.show()

    def _get_agency(self):
        return Agency()

    def _get_settings_validator(self, param):
        return EditSettingsForm(**param)

    def show(self):
        c.form_errors = {}
        c.form_result = {}
        c.agency = self._get_agency()
        return render('/settings/show.mako')

    @checkRole('admin')
    def edit(self):
        c.agency = self._get_agency()
        c.form_errors = {}
        c.form_result = c.agency.conf
        form = render('/settings/edit.mako')
        return render_htmlfill(form, defaults=c.form_result,
                               auto_insert_errors=False,
                               errors=c.form_errors)

    @checkRole('admin')
    @restrict_to_POST
    def editAction(self):
        param = {"max_speicherdauer":
                 MaxMinInt(min=g.mpuls_config.get('case', 'remindermindays'),
                           max=g.mpuls_config.get('case', 'remindermaxdays'),
                           not_empty=True)
                 }

        validator = self._get_settings_validator(param)
        c.agency = self._get_agency()
        try:
            c.form_result = validator.to_python(request.params)
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            c.form_error_msg = error
            form = render('/settings/edit.mako')
            return render_htmlfill(form, defaults=c.form_result,
                                   auto_insert_errors=False,
                                   errors=c.form_errors)

        # Store the values.
        self.pre_save_hook(c.form_result)
        c.agency.store(c.form_result)

        c.dialog_title = _('Configurations saved!')
        c.dialog_text = _('Please click on ok to continue.')
        c.url_ok = url(controller='settings', action='index')
        return render('/settings/dialogs/success_edit.mako')

    def pre_save_hook(self, form_data):
        """
        Customizable function, is called before saving the agency settings
        """
        pass


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
