# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging
import formencode

from mpulsweb.lib.translation import _

from mpulsweb.lib.base import BaseController, c, h, g, render, request, \
     session, restrict_to_POST

from mpulsweb.lib.validators import BaseFormValidator, NewPasswordForm, \
     CheckedNewPasswordForm
from mpulsweb.lib.security import checkRole

from mpulsweb.model.user import UserListObject, UserObject, UserGroupList, \
     UserGroup


log = logging.getLogger(__name__)


class UsersettingsController(BaseController):

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    def index(self):
        # Return a rendered template
        #   return render('/some/template.mako')
        # or, Return a response
        return 'Hello, User'

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    def showSettings(self):
        c.uo = UserObject(h.getUserId())
        c.vuo = UserGroup(c.uo.standin)
        return render('/usersettings/show_settings.mako')

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    def changePassword(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/usersettings/change_password.mako')

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    @restrict_to_POST
    def changePasswordAction(self):

        # Disable changing passwords if administration module is disabled
        if not g.mpuls_config.is_enabled('module', 'administration'):
            return self.changePassword()

        validator = CheckedNewPasswordForm()
        try:
            user = session['USER_AUTHORIZED']
            c.uo = UserObject(user.id)
            form_result = validator.to_python(request.params, user)
            new_pass = c.uo.setPassword(form_result.get('passwd'))
            if new_pass:
                # Save new password in the session
                user.password = new_pass
                session['USER_AUTHORIZED'] = user
                session.save()
                c.dialog_title = _(u"User password changed!")
                c.dialog_text = _(u"You password was successfully changed") + \
                        " " + _("Please click OK to continue.")
                c.url_ok = h.url(controller="usersettings",
                                     action="showSettings")
            else:
                return "Password could not be changed"
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/usersettings/change_password.mako')
        return render('/usersettings/dialogs/notificate_setpassword.mako')

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    def changePasswordOnLogin(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/usersettings/change_password_login.mako')

    @checkRole(('admin', 'cm', 'pt_dlr', 'pb'))
    @restrict_to_POST
    def changePasswordOnLoginAction(self):
        # Do not forget to unset the flag for the need for pass change if this
        # action succseeds
        validator = NewPasswordForm()
        try:
            form_result = validator.to_python(request.params)
            uo = UserObject(h.getUserId())
            new_pass = uo.setPassword(form_result.get('passwd'))
            if new_pass:
                # Save new password in the session
                session_user = session['USER_AUTHORIZED']
                session_user.password = new_pass
                session_user.needs_passrefresh = False
                session['USER_AUTHORIZED'] = session_user
                session.save()
                c.dialog_title = _(u"User password changed!")
                c.dialog_text = _(u"You password was successfully changed") + \
                        " " + _("Please click OK to continue.")
                c.url_ok = h.url(controller="usersettings",
                                     action="showSettings")
            else:
                return "Password could not be changed"
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/usersettings/change_password_login.mako')
        return render('/usersettings/dialogs/notificate_setpassword.mako')

    @checkRole('cm')
    def editStandin(self):
        c.form_errors = {}
        c.form_result = {}
        ulo = UserListObject()
        ug = UserGroupList()
        user = session['USER_AUTHORIZED']
        c.uo = UserObject(user.id)
        # Get available users for standin. filter out admins and the current
        # user
        filter = [u.gid for u in ulo.getAdminList()]
        filter.append(user.gid)
        c.grouplist = ug.getGroupList(filter)
        return render('usersettings/edit_standin.mako')

    @checkRole('cm')
    @restrict_to_POST
    def editStandinAction(self):
        validator = BaseFormValidator()
        try:
            c.form_result = validator.to_python(request.params)
            c.uo = UserObject(h.getUserId())
            c.uo.setStandin(c.form_result.get('standin') or None)
            #TODO: Update the value for the logged in session user
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('usersettings/edit_standin.mako')
        c.dialog_title = _('Standin saved!')
        c.dialog_text =  _('Your standin is saved. Please click on ok to continue.')
        c.url_ok = h.url(controller='usersettings', action='showSettings')
        return render ('/usersettings/dialogs/success_edit_standin.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
