-- Attention!
-- Before executing this script the master table and the repeat groups must
-- have been created. The mpuls-dbadmin script should take care of this!

BEGIN;

-- CREATE PROCEDURAL LANGUAGE plpgsql;

SET search_path = :mpulsschema, pg_catalog;

-- PL/Python ist eine 'untrusted' Sprache
SET ROLE postgres;

-- Modifikations on ks_status_tbl
-- Set FK-Constraint for ka_status_tbl
ALTER TABLE ka_status_tbl ADD CONSTRAINT has_status_master_fk FOREIGN KEY
(master_id) REFERENCES master_tbl (id) ON DELETE CASCADE;

-- Create trigger
CREATE TRIGGER t1_master_tbl_update AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_lastaccess_log();

CREATE TRIGGER t1_ka_status_tbl_update AFTER
    UPDATE ON ka_status_tbl
    FOR EACH ROW EXECUTE PROCEDURE status_lastaccess_log();


-- Generiert eine UUID
--
CREATE OR REPLACE FUNCTION generate_uuidgen()
RETURNS   text AS
$BODY$
    from commands import getstatusoutput
    from random   import randint
    try:
        result = getstatusoutput("/usr/bin/uuid -v4")
        if result[0] != 0: raise Exception()
        return result[1].strip()
    except:
        count = 500
        while count > 0:
            nid = "%08x-%04x-%04x-%04x-%012x" % (
                randint(0, 1 <<  8*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 << 12*4))
            if len(nid) == 36:
                return nid
            count -= 1
        raise Exception("UUID generator failed")
$BODY$
LANGUAGE 'plpythonu' VOLATILE;

REVOKE ALL ON FUNCTION generate_uuidgen() FROM public;
GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :admin_group;
ALTER FUNCTION generate_uuidgen() OWNER TO :adm_ka_owner;

SET ROLE :adm_ka_owner;
BEGIN;

-- Functions
--
-- Funktionen zum Setzen der Einverstaendniserkärung
CREATE OR REPLACE FUNCTION set_ee_status(mid integer, status integer)
    RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
BEGIN
    UPDATE master_tbl_view SET einwilligung = status WHERE id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION set_ee_status(integer, integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION set_ee_status(integer, integer) TO GROUP :cm_group;
GRANT  EXECUTE ON FUNCTION set_ee_status(integer, integer) TO GROUP :admin_group;
--
-- Setzt in der Tabelle 't_name' in der Spalte mit der
-- ID 'id' das Feld 'uuid_id' mit einer UUID.
-- Bei Verletzung des Unique-Constraints wird bis
-- zu 500 mal neu gewuerfelt.
--
CREATE OR REPLACE FUNCTION set_uuid(t_name text, id int)
RETURNS void AS
$$
DECLARE
    xuuid_id char(36);
BEGIN
    FOR i IN 0..500 LOOP
        SELECT INTO xuuid_id generate_uuidgen();
        BEGIN
            EXECUTE 'UPDATE '           || t_name   ||
                    ' SET uuid_id = ''' || xuuid_id ||
                    ''' WHERE id = '    || id;
        EXCEPTION WHEN unique_violation THEN
            CONTINUE;
        END;
        RETURN;
    END LOOP;
    RAISE EXCEPTION 'set_uuid failed';
END;
$$ LANGUAGE plpgsql;
REVOKE ALL ON FUNCTION set_uuid(text, int) FROM public;


-- User ID des Nutzers dem neu angelegte Fälle gehören.
--
-- Normalerweise ist dies identisch zu getuserid. Ausprägungen von MPuls
-- können aber anderes festlegen, indem sie diese Funktion anders
-- definieren.
CREATE OR REPLACE FUNCTION initial_master_ds_owner_id() RETURNS integer
    LANGUAGE 'plpgsql' STABLE
    AS $$
BEGIN
    RETURN getuserid();
END;
$$;

REVOKE ALL ON FUNCTION initial_master_ds_owner_id() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION initial_master_ds_owner_id() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION initial_master_ds_owner_id() TO GROUP :admin_group;


--
-- Funktionen zum Anlegen neuer (Teile von) Fallakten
-- Diese sind nur fuer die Gruppe CM_KA aufrufbar (Admin_KA darf keine neuen
-- Akten anlegen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION create_master_ds(nuuid char(36))
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    ownerid integer;
    standin integer;
    mid     integer;
BEGIN
    ownerid := initial_master_ds_owner_id();
    INSERT INTO master_tbl (bearbeiter_id) VALUES (ownerid);
    SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = ownerid;

    IF standin IS NOT NULL THEN
        INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id)
                VALUES (standin, currval('master_tbl_id_seq'));
    END IF;

    SELECT INTO mid currval('master_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE master_tbl SET uuid_id = nuuid WHERE id = mid;
    ELSE
        PERFORM set_uuid('master_tbl', mid);
    END IF;

    PERFORM create_status_ds(mid);
    PERFORM create_meta_ds(mid);
    RETURN mid;
END;
$$;

REVOKE ALL ON FUNCTION create_master_ds(char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_master_ds(char(36))
    TO GROUP :cm_group;

--
-- Funktion, die ueberprueft, ob eine Kundennummer schon existiert.
CREATE OR REPLACE FUNCTION case_exists(knummer varchar(100))
    RETURNS boolean
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    found boolean;
BEGIN
    SELECT INTO found TRUE FROM master_tbl WHERE kundennummer = knummer;
    IF found THEN
        RETURN TRUE;
    ELSE
        RETURN FALSE;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION case_exists(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION case_exists(varchar(100)) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION case_exists(varchar(100)) TO GROUP :cm_group;

-- Funktionen zum Loeschen einer Fallakte inklusive aller angehaengten
-- Unterstuetzungsangebote, Dateien Termine etc.
-- Diese sind nur fuer die Gruppe ADMIN_KA aufrufbar (CM_KA sollen keine Akten
-- loeschen koennen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION delete_master_ds(mid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    st integer;
BEGIN
    SELECT INTO st s.status FROM master_tbl m JOIN ka_status_tbl s
        ON s.master_id = m.id WHERE m.id = mid;

    IF NOT st in (3, 5) THEN
        RAISE EXCEPTION 'case is not deletable';
    END IF;

    DELETE FROM master_tbl WHERE id = mid;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_master_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_master_ds(integer) TO GROUP :admin_group;

-- Wenn eine Akte anonymisiert ist, hat sie keine UUID mehr. Folglich
-- Kann man aus der anonymized_uuids auch nichts mehr loeschen.
-- Falls sich dort trotzdem ein gleicher Eintrag finden sollte, ist
-- dies eine potentielle Quelle fuer eine Inkonsistenz.
CREATE OR REPLACE FUNCTION delete_anonymized_uuid() RETURNS TRIGGER AS $$
BEGIN
    -- Diese _sollte_ nichts loeschen!
    DELETE FROM anonymized_uuids WHERE uuid_id = OLD.uuid_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER t_delete_anonymized_uuid
    AFTER DELETE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE delete_anonymized_uuid();

-- Dies Funktion 'delete_old_anonymized_cases' loescht alle anonymisierten
-- Fallakten, die laenger als die Aufbewahrungsfrist der jeweiligen KA
-- unbearbeitet im System liegen.
CREATE OR REPLACE FUNCTION delete_old_anonymized_cases()
RETURNS void AS
$$
DECLARE
    mid          int;
    max_age      interval;
    max_age_text text;
BEGIN
    SELECT INTO max_age_text wert FROM ka_konfiguration_tbl_view WHERE bez = 'max_speicherdauer';
    IF NOT FOUND
    THEN
        RETURN;
    END IF;

    max_age = (max_age_text || ' day')::interval;

    FOR mid IN SELECT m.id FROM master_tbl_view m
               JOIN ka_status_tbl_view s
               ON m.id = s.master_id
               WHERE
               s.status = 5 AND age(s.zugriff) >= max_age
    LOOP
        PERFORM delete_master_ds(mid);
    END LOOP;
END;
$$ LANGUAGE plpgsql;

REVOKE ALL    ON FUNCTION delete_old_anonymized_cases() FROM public;
GRANT EXECUTE ON FUNCTION delete_old_anonymized_cases() TO GROUP :admin_group;

-- Haeufige Abfragen, die sowohl von der Gruppe CM_KA, als auch Admin_KA
-- ausgefuehrt werden koennen.

-- gettotal()
-- Anzahl aller Faelle des Fallmanagers oder der KA (wenn Admin)
CREATE OR REPLACE FUNCTION gettotal() RETURNS integer
    LANGUAGE 'plpgsql'
    AS $$
DECLARE
    total int4;
BEGIN
    SELECT INTO total count(id) FROM master_tbl_eval_total_view;
    RETURN total;
END;
$$ ;

REVOKE ALL ON FUNCTION gettotal() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION gettotal() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION gettotal() TO GROUP :admin_group;

-- gettotalcm()
-- Anzahl aller Faelle des Fallmanagers oder der KA (wenn Admin),
-- die sich im Case-Management befinden.
--
CREATE OR REPLACE FUNCTION gettotalcm() RETURNS integer
    LANGUAGE 'plpgsql'
    AS $$
DECLARE
    total_cm int4;
BEGIN
    SELECT INTO total_cm count(id) FROM master_tbl_eval_incm_view;
    RETURN total_cm;
END;
$$ ;

REVOKE ALL ON FUNCTION gettotalcm() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION gettotalcm() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION gettotalcm() TO GROUP :admin_group;

-- get_masterid_from_uuid(myrel varchar, uuid char)
-- Author: Torsten Irlaender
--
-- Gibt die master id des datensatzes mit der übergebenen uuid in der
-- angegebenen realtion wieder. Wenn ein solcher datensatz nicht existiert, wird
-- NULL zurueck gegeben.
--

CREATE OR REPLACE FUNCTION get_masterid_from_uuid(myrel varchar(128), uuid char(36))
    RETURNS integer AS
$$
DECLARE
    myid integer;
BEGIN
    IF myrel = 'master' THEN
        SELECT INTO myid id from master_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'logbuch' THEN
        SELECT INTO myid master_id from ka_logbuch_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'dokumente' THEN
        SELECT INTO myid master_id from ka_dokumente_tbl WHERE uuid_id = uuid;
    END IF;
    RETURN myid;
END;
$$ LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER;
REVOKE ALL ON FUNCTION get_masterid_from_uuid(varchar, char) FROM public;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :admin_group;

-- Constarints zur Sicherstellung das Jede Fallakte auch einen Bearbeiter hat.
ALTER TABLE nm_benutzer_master_tbl ADD CONSTRAINT has_master_fk FOREIGN KEY
(master_id) REFERENCES master_tbl (id) ON DELETE CASCADE;

ALTER TABLE master_tbl ADD CONSTRAINT has_bearbeiter_id_fk FOREIGN KEY
(bearbeiter_id) REFERENCES ka_benutzer_tbl;

-- Trigger/Logging
CREATE TRIGGER t_master_tbl_insert AFTER
    INSERT ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log(); 

CREATE TRIGGER t_master_tbl_update AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t_master_tbl_delete AFTER
    DELETE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

-- Trigger, der nach Uebertragung der Fallakte an einen anderen
-- Bearbeiter den alten und neuen Bearbeiter eintraegt in das Logbuch eintraegt.
--

CREATE OR REPLACE FUNCTION master_ds_editor_changed() RETURNS trigger AS $master_ds_editor_changed$
DECLARE
    lid      integer;
    old_user RECORD;
    new_user RECORD;
BEGIN
    IF OLD.bearbeiter_id <> NEW.bearbeiter_id THEN

        SELECT INTO old_user * FROM ka_benutzer_tbl WHERE id = OLD.bearbeiter_id;
        SELECT INTO new_user * FROM ka_benutzer_tbl WHERE id = NEW.bearbeiter_id;

        SELECT INTO lid create_logbuch_ds(NEW.id, NULL);

        UPDATE ka_logbuch_tbl_view
            SET art = 0, 
            notiz = notiz_from_plaintext('Fallakte übertragen von ' 
                    || coalesce(old_user.nachname, '-/-') || ', '
                    || coalesce(old_user.vorname,  '-/-') || ' an '
                    || coalesce(new_user.nachname, '-/-') || ', '
                    || coalesce(new_user.vorname,  '-/-'))
        WHERE
            id = lid;
    END IF;
    RETURN NEW;
END;
$master_ds_editor_changed$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_editor_changed_trigger AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_editor_changed();

COMMIT;
