#!/usr/bin/env python

import sys
from optparse import OptionParser
from datetime import timedelta

import psycopg2 as dbapi

try:
    # Python >= 2.5 includes SQLite 3
    import sqlite3 as db
except ImportError:
    # installed separately in earlier versions
    from pysqlite2 import dbapi2 as db

DEFAULT_PORT = 5436

LOAD_JP_FROM_FKZ = """SELECT database, job_percentage FROM fkz_tbl;"""
UPDATE_JP_IN_FKZ = """UPDATE fkz_tbl SET job_percentage = %(jobpercentage)s
                      WHERE database = %(database)s;"""
def usage(msg="", exit_code=0):
    print >> sys.stderr, """%s
usage:
%s -i <sqlite db> -o <postgres db> [-p <port>]
""" % (msg, sys.argv[0])
    sys.exit(exit_code)

def update_jp_in_fkz(in_cur, out_cur):
    # Get the data out of the fkz table
    in_cur.execute(LOAD_JP_FROM_FKZ)
    field_list = []
    for row in in_cur.fetchall():
        fields = {'database': row[0], 'jobpercentage': row[1]}
        field_list.append(fields)

    # Update the out fkz table with these data
    for f in field_list:
        out_cur.execute(UPDATE_JP_IN_FKZ, f)

def main():
    parser = OptionParser()
    parser.set_defaults(port=DEFAULT_PORT)
    parser.add_option("-i", "--in", dest="indb",
                      help="Name of input database file")
    parser.add_option("-o", "--out", dest="outdb",
                      help="Name of output PostgreSQL database")
    parser.add_option("-p", "--port", type="int",
                      help="Port number")

    opts, args = parser.parse_args()

    if not opts.indb:
        usage("error: missing input db", 1)
    if not opts.outdb:
        usage("error: missing output db", 1)
    if not opts.port:
        usage("error: missing port", 1)

    in_con = in_cur = out_con = out_cur = None
    try:
        in_con = db.connect(opts.indb)
        in_cur = in_con.cursor()

        out_con = dbapi.connect(database=opts.outdb, port=opts.port)
        out_cur = out_con.cursor()

        try:
            update_jp_in_fkz(in_cur, out_cur)
            out_con.commit()
        except:
            out_con.rollback()
            raise
    finally:
        for c in (in_cur, in_con, out_cur, out_con):
            if c:
                c.close()


if __name__ == '__main__':
    main()
