# -*- coding: utf-8 -*-
# Copyright (C) 2011 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>

import unittest

from formed.meta.data import RepeatNode


class RepeatNodeTest(unittest.TestCase):

    def new_repeat_node(self, **attrs):
        node = RepeatNode()
        for key, value in attrs.iteritems():
            node.setAttribute(key, value)
        return node

    def test_get_orderby_single_field(self):
        node = self.new_repeat_node(orderby="start")
        self.assertEquals(node.getOrderBy(), [("start", "asc")])

    def test_get_orderby_multiple_fields(self):
        node = self.new_repeat_node(orderby="start, end")
        self.assertEquals(node.getOrderBy(), [("start", "asc"), ("end", "asc")])

    def test_get_orderby_multiple_fields_direction(self):
        node = self.new_repeat_node(orderby="start desc, end asc")
        self.assertEquals(node.getOrderBy(),
                          [("start", "desc"), ("end", "asc")])

    def test_get_orderby_single_colum_reverse(self):
        node = self.new_repeat_node(orderby="start", reverse="true")
        self.assertEquals(node.getOrderBy(), [("start", "desc")])

    def test_get_orderby_digest_fallback(self):
        node = self.new_repeat_node(digest="lastname firstname")
        self.assertEquals(node.getOrderBy(), [("lastname", "asc")])

    def test_get_orderby_multiple_fields_reverse(self):
        node = self.new_repeat_node(orderby="lastname, firstname",
                                    reverse="true")
        self.assertRaises(ValueError, node.getOrderBy)

    def test_get_orderby_single_field_desc_reverse(self):
        node = self.new_repeat_node(orderby="lastname desc", reverse="true")
        self.assertRaises(ValueError, node.getOrderBy)
