#!/usr/bin/python
# -*- coding: utf-8 -*-
"""\n
This is a comment

Usage: evaltags [options] configfile 

Options:
    -h / --help     Print this message and exit.
    -v              Print sql statements which are sent to the database

    Database credentials. Defaults are set in db/db.py

    -u              username
    -P              password
    -d              database
    -H              host
    -p              port

    Tagging options

    --caseid        Commaseprated list of ids which will be checks. Defaults to
                    all cases in the database

"""
# 20060403 Torsten Irlaender <torsten.irlaender@intevation.de>
#
# This program is free software under the GNU GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys 
import os
import getopt
import logging
from libmpuls.service.tags import TagSetterConfig, TagSetter
from libmpuls.db.db import CONFIG

logging.basicConfig()


def usage(code, msg=''):
    print >> sys.stderr, __doc__
    if msg:
        print >> sys.stderr, msg
    sys.exit(code)

def main():
    try:
        opts, args = getopt.getopt(
            sys.argv[1:], 
            'hvu:P:d:H:p:',
            ['help', 
            'verbose',
            'caseid='])
    except getopt.error, msg:
        usage(1, msg)

    username = CONFIG.get('login')
    password = CONFIG.get('password')
    database = CONFIG.get('dbname')
    host     = CONFIG.get('url')
    port     = CONFIG.get('port')
    caseid   = None
    verbose  = False

    for opt, arg in opts:
        if opt in ('-h', '--help'):
            usage(0)
        if opt in ('-u'):
            username = arg
        if opt in ('-P'):
            password = arg
        if opt in ('-d'):
            database = arg
        if opt in ('-H'):
            host = arg
        if opt in ('-p'):
            port = arg
        if opt in ('--caseid'):
            caseid = arg
        if opt in ('--verbose'):
            verbose = True
    if len(args) < 1:
        usage(1)

    dbname = CONFIG.get('dbname') 

    config = TagSetterConfig(args[0], username, password, database, host, port, caseid)
    setter = TagSetter(config)
    setter.perform()

if __name__ == '__main__':
    main()
